@@ -4,6 +4,8 @@ use strict;
 
 use Graphics::Framebuffer;
 use List::Util qw(min max);
+use Time::HiRes qw(sleep time);
+# use Data::Dumper::Simple; $Data::Dumper::Sortkeys=1;
 
 system('clear');
 $| = 1;
@@ -14,6 +16,9 @@ my $new_x;
 my $new_y;
 my $F;
 my $dev = 0;
+my $s;
+my $image;
+my $angle;
 
 $file = 1 if ($arg =~ /file/i);    # File Handle drawing mode
 if ($arg =~ /(\d+)/) {
@@ -47,8 +52,9 @@ my $center_y      = $YY / 2;
 my $factor        = 5;
 
 $F->cls('OFF');
+
 print_it($F, 'Plotting Test');
-my $s = time + $factor;
+$s = time + $factor;
 while (time < $s) {
     my $x = int(rand($screen_width));
     my $y = int(rand($screen_height));
@@ -65,10 +71,10 @@ while (time < $s) {
 
 print_it($F, 'Angle Line Test');
 $s = time + $factor;
-my $angle = 0;
+$angle = 0;
 while (time < $s) {
     $F->set_color({'red' => rand(256), 'green' => rand(256), 'blue' => rand(256)});
-    $F->angle_line({'x' => $center_x, 'y' => $center_y, 'radius' => $center_y, 'angle' => $angle++});
+    $F->angle_line({'x' => $center_x, 'y' => $center_y, 'radius' => ($center_y - $F->{'Y_CLIP'}), 'angle' => $angle++});
     $angle = 0 if ($angle >= 360);
 } ## end for (my $l = $F->{'Y_CLIP'...})
 
@@ -93,22 +99,22 @@ while (time < $s) {
     $F->box(
         {
             'x'        => rand($XX),
-            'y'        => rand($YY),
-            'xx'       => rand($XX),
-            'yy'       => rand($YY),
-            'filled'   => 1,
-            'gradient' => {
-                'start' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                },
-                'end' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                }
-            }
+              'y'        => rand($YY),
+              'xx'       => rand($XX),
+              'yy'       => rand($YY),
+              'filled'   => 1,
+              'gradient' => {
+                  'start' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  },
+                  'end' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  }
+              }
         }
     );
 } ## end while (time < $s)
@@ -119,23 +125,23 @@ while (time < $s) {
     $F->box(
         {
             'x'        => rand($XX),
-            'y'        => rand($YY),
-            'xx'       => rand($XX),
-            'yy'       => rand($YY),
-            'filled'   => 1,
-            'gradient' => {
-                'direction' => 'horizontal',
-                'start' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                },
-                'end' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                }
-            }
+              'y'        => rand($YY),
+              'xx'       => rand($XX),
+              'yy'       => rand($YY),
+              'filled'   => 1,
+              'gradient' => {
+                  'direction' => 'horizontal',
+                  'start' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  },
+                  'end' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  }
+              }
         }
     );
 } ## end while (time < $s)
@@ -161,23 +167,23 @@ while (time < $s) {
     $F->box(
         {
             'x'        => rand($XX),
-            'y'        => rand($YY),
-            'xx'       => rand($XX),
-            'yy'       => rand($YY),
-            'radius'   => rand($XX / 16),
-            'filled'   => 1,
-            'gradient' => {
-                'start' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                },
-                'end' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                }
-            }
+              'y'        => rand($YY),
+              'xx'       => rand($XX),
+              'yy'       => rand($YY),
+              'radius'   => rand($XX / 16),
+              'filled'   => 1,
+              'gradient' => {
+                  'start' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  },
+                  'end' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  }
+              }
         }
     );
 } ## end while (time < $s)
@@ -189,24 +195,24 @@ while (time < $s) {
     $F->box(
         {
             'x'        => rand($XX),
-            'y'        => rand($YY),
-            'xx'       => rand($XX),
-            'yy'       => rand($YY),
-            'radius'   => rand($XX / 16),
-            'filled'   => 1,
-            'gradient' => {
-                'direction' => 'horizontal',
-                'start' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                },
-                'end' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                }
-            }
+              'y'        => rand($YY),
+              'xx'       => rand($XX),
+              'yy'       => rand($YY),
+              'radius'   => rand($XX / 16),
+              'filled'   => 1,
+              'gradient' => {
+                  'direction' => 'horizontal',
+                  'start' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  },
+                  'end' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  }
+              }
         }
     );
 } ## end while (time < $s)
@@ -232,21 +238,21 @@ while (time < $s) {
     $F->circle(
         {
             'x'        => rand($XX),
-            'y'        => rand($YY),
-            'radius'   => rand($center_y),
-            'filled'   => 1,
-            'gradient' => {
-                'start' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                },
-                'end' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                }
-            }
+              'y'        => rand($YY),
+              'radius'   => rand($center_y),
+              'filled'   => 1,
+              'gradient' => {
+                  'start' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  },
+                  'end' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  }
+              }
         }
     );
 } ## end while (time < $s)
@@ -258,22 +264,22 @@ while (time < $s) {
     $F->circle(
         {
             'x'        => rand($XX),
-            'y'        => rand($YY),
-            'radius'   => rand($center_y),
-            'filled'   => 1,
-            'gradient' => {
-                'direction' => 'horizontal',
-                'start' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                },
-                'end' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                }
-            }
+              'y'        => rand($YY),
+              'radius'   => rand($center_y),
+              'filled'   => 1,
+              'gradient' => {
+                  'direction' => 'horizontal',
+                  'start' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  },
+                  'end' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  }
+              }
         }
     );
 } ## end while (time < $s)
@@ -306,22 +312,22 @@ while (time < $s) {
     $F->filled_pie(
         {
             'x'             => rand($XX),
-            'y'             => rand($YY),
-            'radius'        => rand($center_y),
-            'start_degrees' => rand(360),
-            'end_degrees'   => rand(360),
-            'gradient'      => {
-                'start' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                },
-                'end' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                }
-            }
+              'y'             => rand($YY),
+              'radius'        => rand($center_y),
+              'start_degrees' => rand(360),
+              'end_degrees'   => rand(360),
+              'gradient'      => {
+                  'start' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  },
+                  'end' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  }
+              }
         }
     );
 } ## end while (time < $s)
@@ -347,22 +353,22 @@ while (time < $s) {
     $F->ellipse(
         {
             'x'        => rand($XX),
-            'y'        => rand($YY),
-            'xradius'  => rand($center_x),
-            'yradius'  => rand($center_y),
-            'filled'   => 1,
-            'gradient' => {
-                'start' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                },
-                'end' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                }
-            }
+              'y'        => rand($YY),
+              'xradius'  => rand($center_x),
+              'yradius'  => rand($center_y),
+              'filled'   => 1,
+              'gradient' => {
+                  'start' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  },
+                  'end' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  }
+              }
         }
     );
 } ## end while (time < $s)
@@ -374,23 +380,23 @@ while (time < $s) {
     $F->ellipse(
         {
             'x'        => rand($XX),
-            'y'        => rand($YY),
-            'xradius'  => rand($center_x),
-            'yradius'  => rand($center_y),
-            'filled'   => 1,
-            'gradient' => {
-                'direction' => 'horizontal',
-                'start' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                },
-                'end' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                }
-            }
+              'y'        => rand($YY),
+              'xradius'  => rand($center_x),
+              'yradius'  => rand($center_y),
+              'filled'   => 1,
+              'gradient' => {
+                  'direction' => 'horizontal',
+                  'start' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  },
+                  'end' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  }
+              }
         }
     );
 } ## end while (time < $s)
@@ -416,19 +422,19 @@ while (time < $s) {
     $F->polygon(
         {
             'coordinates' => [rand($XX), rand($YY), rand($XX), rand($YY), rand($XX), rand($YY), rand($XX), rand($YY)],
-            'filled'      => 1,
-            'gradient'    => {
-                'start' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                },
-                'end' => {
-                    'red'   => rand(256),
-                    'green' => rand(256),
-                    'blue'  => rand(256)
-                }
-            }
+              'filled'      => 1,
+              'gradient'    => {
+                  'start' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  },
+                  'end' => {
+                      'red'   => rand(256),
+                      'green' => rand(256),
+                      'blue'  => rand(256)
+                  }
+              }
         }
     );
 } ## end while (time < $s)
@@ -458,13 +464,13 @@ while (time < $s) {
         my $b = $F->ttf_print(
             {
                 'x'            => $x,
-                'y'            => $y,
-                'height'       => $h,
-                'wscale'       => $ws,
-                'color'        => sprintf('%02x%02x%02x', int(rand(256)), int(rand(256)), int(rand(256))),
-                'text'         => 'GFB',
-                'bounding_box' => 1,
-                'center'       => 3
+                  'y'            => $y,
+                  'height'       => $h,
+                  'wscale'       => $ws,
+                  'color'        => sprintf('%02x%02x%02x%02x', int(rand(256)), int(rand(256)), int(rand(256)),int(rand(256))),
+                  'text'         => 'GFB',
+                  'bounding_box' => 1,
+                  'center'       => 3
             }
         );
         if (defined($b)) {
@@ -511,11 +517,11 @@ while (time < $s) {
     $F->replace_color(
         {
             'old_red'   => $r,
-            'old_green' => $g,
-            'old_blue'  => $b,
-            'new_red'   => $R,
-            'new_green' => $G,
-            'new_blue'  => $B
+              'old_green' => $g,
+              'old_blue'  => $b,
+              'new_red'   => $R,
+              'new_green' => $G,
+              'new_blue'  => $B
         }
     );
     ($r, $g, $b) = ($R, $G, $B);
@@ -538,11 +544,11 @@ while (time < $s) {
     $F->replace_color(
         {
             'old_red'   => $r,
-            'old_green' => $g,
-            'old_blue'  => $b,
-            'new_red'   => $R,
-            'new_green' => $G,
-            'new_blue'  => $B
+              'old_green' => $g,
+              'old_blue'  => $b,
+              'new_red'   => $R,
+              'new_green' => $G,
+              'new_blue'  => $B
         }
     );
     ($r, $g, $b) = ($R, $G, $B);
@@ -555,36 +561,37 @@ while (time < $s) {
     my $image = $F->load_image(
         {
             'x'      => 0,
-            'y'      => 0,
-            'width'  => rand($XX),
-            'height' => rand($YY),
-            'file'   => 'images/testpattern.jpg'
+              'y'      => 0,
+              'width'  => rand($XX),
+              'height' => rand($YY),
+              'center' => 3,
+              'file'   => 'images/testpattern.jpg'
         }
     );
     $F->blit_write($image);
 } ## end while (time < $s)
 
-my $image = $F->load_image(
+$image = $F->load_image(
     {
         'x'      => 0,
-        'y'      => 0,
-        'width'  => $XX,
-        'height' => $YY,
-        'file'   => 'images/testpattern.jpg'
+          'y'      => 0,
+          'width'  => $XX,
+          'height' => $YY,
+          'file'   => 'images/testpattern.jpg'
     }
 );
 $F->blit_write($image);
-sleep $factor;
+sleep $factor/2;
 
 print_it($F, 'Testing Image blitting');
 
 $image = $F->load_image(
     {
         'x'      => 0,
-        'y'      => 0,
-        'width'  => min(640, $XX / 2),
-        'height' => min(405, $YY / 2),
-        'file'   => 'images/RWBY_Faces.png'
+          'y'      => 0,
+          'width'  => min(600, $XX / 2),
+          'height' => min(405, $YY / 2),
+          'file'   => 'images/RWBY_Faces.png'
     }
 );
 $s = time + $factor;
@@ -593,8 +600,46 @@ while (time < $s) {
     $image->{'y'} = abs(rand($YY - $image->{'height'}));
     $F->blit_write($image);
 }
+
+
+$angle = 0;
+$s = time + $factor;
+while (time < $s) {
+    my $rot = $F->blit_transform(
+        {
+            'rotate' => {
+                'degrees' => $angle
+            },
+              'blit_data' => $image
+        }
+    );
+    $rot->{'x'} = abs(($XX - $rot->{'width'})/2);
+    $rot->{'y'} = abs(($YY - $rot->{'height'})/2);
+    print_it($F, "Testing Image rotate $angle degrees");
+    $F->blit_write($rot);
+    $angle+=10;
+    $angle = 0 if ($angle >= 360);
+}
+
+$s = time + $factor;
+while (time < $s) {
+    foreach my $dir (qw(normal horizontal normal vertical)) {
+        my $rot = $F->blit_transform(
+            {
+                'flip'      => $dir,
+                  'blit_data' => $image
+            }
+        );
+        $rot->{'x'} = abs(($XX - $rot->{'width'})/2);
+        $rot->{'y'} = abs(($YY - $rot->{'height'})/2);
+        print_it($F, "Testing Image flip $dir");
+        $F->blit_write($rot);
+        sleep .1;
+    }
+}
 # sleep $factor;
 
+
 print_it($F, 'Testing Monochrome Image blitting');
 
 $image->{'image'} = $F->monochrome({'image' => $image->{'image'}, 'bits' => $F->{'BITS'}});
@@ -605,6 +650,35 @@ while (time < $s) {
     $F->blit_write($image);
 }
 
+print_it($F, 'Testing Animated Image Display');
+my @list = (
+    [502,242,'RWBY_Kick_Butt.gif'],
+    [402,227,'RWBY_Yang_Kick_Butt.gif'],
+    [322,182,'RWBY_Yang_Bam.gif'],
+);
+foreach my $info (@list) {
+    $image = $F->load_image(
+        {
+            'x'      => 0,
+            'y'      => 0,
+            'width'  => min($XX,shift(@{$info})),
+            'height' => min($YY,shift(@{$info})),
+            'file'   => 'images/' . shift(@{$info}),
+            'center' => 3
+    }
+    );
+    $s = time + ($factor * 2);
+    if (defined($image)) {
+        $F->cls();
+        while (time <= $s) {
+            foreach my $frame (0 .. (scalar(@{$image}) - 1)) {
+                my $blit_time = time;
+                $F->blit_write($image->[$frame]);
+                sleep (($image->[$frame]->{'tags'}->{'gif_delay'} * .01) - (time - $blit_time));
+            }
+        }
+    }
+}
 $F->clip_reset();
 $F->cls('ON');
 
@@ -623,17 +697,18 @@ sub print_it {
         my $b = $fb->ttf_print(
             {
                 'x'            => 5 * $xm,
-                'y'            => max(9, 20 * $ym),
-                'height'       => max(10, 20 * $ym),
-                'color'        => '00FF00',
-                'text'         => $message,
-                'bounding_box' => 1,
-                'center'       => $fb->{'CENTER_X'},
-                'antialias'    => 1
+                  'y'            => max(9, 20 * $ym),
+                  'height'       => max(10, 20 * $ym),
+                  'color'        => '00FF00',
+                  'text'         => $message,
+                  'bounding_box' => 1,
+                  'center'       => $fb->{'CENTER_X'},
+                  'antialias'    => 1
             }
         );
         $fb->ttf_print($b);
         $fb->clip_set({'x' => 0, 'y' => $b->{'pheight'}, 'xx' => $XX, 'yy' => $YY});
+        $center_y = (($YY - $b->{'pheight'}) / 2) + $b->{'pheight'};
         $fb->normal_mode();
     } else {
         print "$message\n";
